import re

from nonebot import require
from nonebot.plugin.on import on_command

require("nonebot_plugin_uninfo")
from nonebot_plugin_uninfo import (
    get_session,
    Uninfo
)
require("nonebot_plugin_alconna")
from nonebot_plugin_alconna import (
    on_alconna,
    AlconnaMatcher,
    Match
)
from arclet.alconna import (
    Alconna,
    Args,
    Subcommand,
    Arparma,
    Option
)
from nonebot_plugin_alconna.uniseg import (
    UniMessage,
    Image
)

from .stats import (
    get_level,
    get_stats_image
)

timewindow_prefix = ["生涯", ""]
name_args = Args["name?", str]


battle_pass = on_alconna(
    Alconna(timewindow_prefix, "季卡", name_args)
)

stats = on_alconna(
    Alconna(timewindow_prefix, '战绩', name_args)
)

@battle_pass.handle()
@stats.handle()
async def _(
    matcher: AlconnaMatcher,
    session: Uninfo,
    name: Match[str]
):
    if name.available:
        matcher.set_path_arg('name', name.result)
        return
    # 获取群昵称
    if not session.member or not session.member.nick:
        return
    pattern = r'(?:id:|id\s)(.+)'
    if match := re.match(
        pattern,
        session.member.nick,
        re.IGNORECASE
    ):
        matcher.set_path_arg('name', match.group(1))
        
        
name_prompt = UniMessage.template("{:At(user, $event.get_user_id())} 请发送游戏名称(群昵称设置为id:name/ID name可快速查询)")

@battle_pass.got_path('name', prompt=name_prompt)
async def _(arp: Arparma, name: str):
    level_info = await get_level(name, arp.header_match.result)
    await battle_pass.finish(level_info)

@stats.got_path('name', prompt=name_prompt)
async def _(arp: Arparma, name: str):
    stats_img = await get_stats_image(name, arp.header_match.result)
    if stats_img.startswith('http'):
        res = await UniMessage(Image(url=stats_img)).export()
    else:
        res = stats_img
    await stats.finish(res)


shop = on_command('商城')

@shop.handle()
async def _():
    await shop.finish('https://www.fortnite.com/item-shop?lang=zh-Hans')


from functools import wraps

def exception_handler():
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            res = None
            try:
                res = func(*args, **kwargs)
            except Exception as e:
                res = e.message
            finally:
                return res
        return wrapper
    return decorator




from nonebot import (
    require,
    get_driver, # @get_driver().on_startup 装饰启动时运行函数
    get_bots    # dict[str, BaseBot]
)
from nonebot.log import logger
from nonebot.plugin import PluginMetadata


require("nonebot_plugin_apscheduler")
from nonebot_plugin_apscheduler import scheduler

from .config import Config
from .matcher import *

__plugin_meta__ = PluginMetadata(
    name="堡垒之夜游戏插件",
    description="堡垒之夜战绩，季卡，商城，vb图查询",
    usage="略",
    type="application",
    config=Config,
    homepage="https://github.com/fllesser/nonebot-plugin-fortnite",
    supported_adapters={ "~onebot.v11" }
)




import httpx
import asyncio

# from pathlib import Path
from .config import fconfig
from .other import exception_handler
from fortnite_api import (
    Client,
    StatsImageType,
    TimeWindow
)

api_key = fconfig.fortnite_api_key

@exception_handler()
async def get_level(name: str, time_window: str) -> int:
    async with Client(api_key=api_key) as client:
        stats = await client.fetch_br_stats(
            name=name,
            time_window=TimeWindow.LIFETIME if time_window else TimeWindow.SEASON
        )
    bp = stats.battle_pass
    return f'等级: {bp.level} 下一级进度: {bp.progress}%'

@exception_handler()
async def get_stats_image(name: str, time_window: str) -> str:
    async with Client(api_key=api_key) as client:
        stats = await client.fetch_br_stats(
            name=name,
            image=StatsImageType.ALL,
            time_window=TimeWindow.LIFETIME if time_window else TimeWindow.SEASON
        )
    return stats.image.url
      

# async def write_cn_name(url: str, nickname: str):
#     # 打开原始图像
#     image = Image.open(IMG_PATH / "zhanji.png")
#     async with httpx.AsyncClient() as client:
#         resp = await client.get(url)
#     im = Image.open(BytesIO(resp.content))
#     draw = ImageDraw.Draw(im)
    
#     # 矩形区域的坐标
#     left, top, right, bottom = 26, 90, 423, 230
#     # 获取渐变色的起始和结束颜色
#     start_color = image.getpixel((left, top))
#     end_color = image.getpixel((right, bottom))
    
#     # 创建渐变色并填充矩形区域
#     width = right - left
#     height = bottom - top
    
#     for i in range(width):
#         for j in range(height):
#             r = int(start_color[0] + (end_color[0] - start_color[0]) * (i + j) / (width + height))
#             g = int(start_color[1] + (end_color[1] - start_color[1]) * (i + j) / (width + height))
#             b = int(start_color[2] + (end_color[2] - start_color[2]) * (i + j) / (width + height))
#             draw.point((left + i, top + j), fill=(r, g, b))

#     font_size = 36
#     hansans = (FONT_PATH / "SourceHanSansSC-Bold-2.otf").absolute()
#     font = ImageFont.truetype(hansans, font_size)
#     length = draw.textlength(nickname, font=font)
#     x = left + (right - left - length) / 2
#     y = top + (bottom - top - font_size) / 2
#     draw.text((x, y), nickname, fill = "#fafafa", font = font)
#     buffered = BytesIO() 
#     im.save(buffered, format="PNG") 
#     return base64.b64encode(buffered.getvalue()).decode()
